from dataclasses import dataclass

import click

from hpcflow.cli import cli
from hpcflow.runtime import RunTimeInfo


@dataclass
class HPCFlow:
    """Class for instantiating an HPCFlow application, which may provide, for instance,
    custom parameter logic, over the top of that provided by hpcflow."""

    name: str
    version: str

    def __post_init__(self):
        self.CLI = self.make_CLI()

    def make_CLI(self):
        def new_CLI(ctx, debug):
            ctx.obj = RunTimeInfo(name=self.name, debug=debug)
            if debug:
                click.echo("Debug mode is ON.")
                click.echo(f"run_time_info is: {ctx.obj!r}.")

        new_CLI = click.version_option(
            package_name=self.name, prog_name=self.name, version=self.version
        )(new_CLI)
        new_CLI = click.option("--debug/--no-debug", default=False)(new_CLI)
        new_CLI = click.pass_context(new_CLI)
        new_CLI = click.group(name=self.name)(new_CLI)

        # add hpcflow CLI as a sub command:
        new_CLI.add_command(cli)
        for name, command in cli.commands.items():
            # add each hpcflow command as a new CLI command:
            new_CLI.add_command(command, name=name)
        return new_CLI



from imp import is_frozen
import os
from pathlib import Path
import sys


class RunTimeInfo:
    """Get the run-time information, including the executable name used to
    invoke the CLI, in the case a PyInstaller-built executable was used."""

    def __init__(self, name, debug=False):

        is_frozen = getattr(sys, "frozen", False)
        bundle_dir = (
            sys._MEIPASS if is_frozen else os.path.dirname(os.path.abspath(__file__))
        )

        self.name = name
        self.debug = debug
        self.is_frozen = is_frozen
        self.working_dir = os.getcwd()

        path_exec = Path(sys.executable)
        path_argv = Path(sys.argv[0])

        if self.is_frozen:
            self.bundle_dir = Path(bundle_dir)
            self.executable_path = path_argv
            self.resolved_executable_path = path_exec
            self.executable_name = self.executable_path.name
            self.resolved_executable_name = self.resolved_executable_path.name
        else:
            self.script_path = path_argv
            self.python_executable_path = path_exec

    def __repr__(self):
        if self.is_frozen:
            return (
                f"{self.__class__.__name__}(name={self.name!r}, debug={self.debug!r}, "
                f"is_frozen={self.is_frozen!r}, "
                f"executable_name={self.executable_name!r}, "
                f"resolved_executable_name={self.resolved_executable_name!r}, "
                f"executable_path={self.executable_path!r}, "
                f"resolved_executable_path={self.resolved_executable_path!r}, "
                f"working_dir={self.working_dir!r}"
                f")"
            )
        else:
            return (
                f"{self.__class__.__name__}(name={self.name!r}, debug={self.debug!r}, "
                f"is_frozen={self.is_frozen!r}, "
                f"script_path={self.script_path!r}, "
                f"python_executable_path={self.python_executable_path!r}, "
                f"working_dir={self.working_dir!r}"
                f")"
            )



import click
from hpcflow import __version__
from hpcflow.runtime import RunTimeInfo


@click.group(name="hpcflow")
@click.pass_context
@click.option("--debug/--no-debug", default=False)
@click.version_option(version=__version__, package_name="hpcflow", prog_name="hpcflow")
def cli(ctx, debug):
    ctx.obj = RunTimeInfo(name="hpcflow", debug=debug)
    if debug:
        click.echo("Debug mode is ON.")
        click.echo(f"run_time_info is: {ctx.obj!r}.")


@cli.command()
def make_workflow():
    """Example command on hpcflow"""
    pass


if __name__ == "__main__":
    cli()
